/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#pragma once
#include "Map2D.h"
#include <list>
#include <algorithm>
#include "PriorityQueue.h"
#include "PathMapTile.h"
#include <hash_map>
#include "Platform.h"
#include "Singleton.h"
#include "IntVector.h"

namespace SLK
{

struct SetFinder
{
	bool operator() (PathMapTile *source, PathMapTile *compare)const
	{
		if(source->x == compare->x)
		{
			if(source->y == compare->y)
			{
				return false;
			}
			else
			{
				return source->y < compare->y;
			}
		}
		else
		{
			return source->x < compare->x;
		}
	}
};

class PathMapTileFinder
{
public:
	PathMapTileFinder(int x, int y):
		m_x(x),
		m_y(y)
	{
	}

	bool operator() (PathMapTile *find)const
	{
		return find->x == m_x && find->y==m_y;
	}
private:
	int m_x;
	int m_y;
};



class SerilynExport AStarPathfind:public Singleton<AStarPathfind>
{
public:
	AStarPathfind(Map2D* map);
	virtual ~AStarPathfind(void);

	//Returns path cost
	float getPath(int startX, int startY, int endX, int endY, std::vector<IntVector> &path);


	static AStarPathfind* getSingletonPtr();
	static AStarPathfind& getSingleton();
private:
	enum MoveType{Straight, NorthWest, NorthEast, SouthWest, SouthEast};

	inline void checkNode(int x, int y, PathMapTile *parent, MoveType moveType);
	inline PathMapTile* getTile(int x, int y);//gets a tile from the pool or creates one if there aren't any

	void constructPath(PathMapTile *endNode, std::vector<IntVector> &path);


	void clear();//adds all tiles in open and closed back to the pool
	inline float getHeuristic(int x, int y)
	{
		return Ogre::Math::Abs((x-m_endX)) + Ogre::Math::Abs((y-m_endY));
	}

	stdext::hash_map<int, PathMapTile*> m_master;
	PriorityQueue m_open;

	std::vector<PathMapTile*> m_pool;
	Map2D *m_map;
	int m_endX;
	int m_endY;
	int m_size;

	int m_tilesCreated;//debug - tiles ever created


};

PathMapTile* AStarPathfind::getTile(int x, int y)
{
	stdext::hash_map<int, PathMapTile*>::iterator iter = m_master.find(x+y*m_size);
	if(iter != m_master.end())
	{
		return iter->second;
	}
	else
	{
		PathMapTile* tile;
		if(m_pool.size() == 0)
		{
			tile = new PathMapTile();
			m_tilesCreated++;
		}
		else
		{
			tile = m_pool.back();
			m_pool.pop_back();
		}
		tile->x = x;
		tile->y = y;
		tile->onClosed = false;
		tile->onOpen = false;
		m_master.insert(std::make_pair(x+y*m_size,tile));
		return tile;
	}	
}

void AStarPathfind::checkNode(int x, int y, PathMapTile *parent, MoveType moveType = Straight)
{
	static PathMapTile newTile;
	newTile.x = x;
	newTile.y = y;

	float cost = m_map->getCost(x,y);
	if(cost == 0)
		return;

	switch(moveType)
	{
	case NorthWest://x-1,y-1
		if(!m_map->canWalk(x+1,y) || !m_map->canWalk(x,y+1))
			return;
		cost *= 1.414f;//multiply it by the extra walk distance
		break;
	case NorthEast://x+1,y-1
		if(!m_map->canWalk(x-1,y) || !m_map->canWalk(x,y+1))
			return;
		cost *= 1.414f;//multiply it by the extra walk distance
		break;
	case SouthWest://x-1,y+1
		if(!m_map->canWalk(x+1,y) || !m_map->canWalk(x,y-1))
			return;
		cost *= 1.414f;//multiply it by the extra walk distance
		break;
	case SouthEast://x+1,y+1
		if(!m_map->canWalk(x-1,y) || !m_map->canWalk(x,y-1))
			return;
		cost *= 1.414f;//multiply it by the extra walk distance
		break;
	}

	//Don't search where we just came from
	if(parent->parent == 0 || !(parent->parent->x == x && parent->parent->y == y))
	{
		newTile.parent = parent;

		newTile.cost = parent->cost + cost;
		newTile.total = newTile.cost + getHeuristic(x,y);

		PathMapTile *actualTile = getTile(x,y);
		if(!(actualTile->onOpen && newTile.total > 
			actualTile->total) && 
			!(actualTile->onClosed  && newTile.total > actualTile->total))
		{
			actualTile->onClosed = false;
			actualTile->parent = newTile.parent;
			actualTile->cost = newTile.cost;
			actualTile->total = newTile.total;

			if(actualTile->onOpen)
			{
				m_open.update(actualTile);
			}
			else
			{
				m_open.push(actualTile);
				actualTile->onOpen = true;
			}
		}
	}
}

}