/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#pragma once
#include "UIManager.h"
#include "Timer.h"
#include "State.h"
#include "Platform.h"
#include "Singleton.h"
#include "SoundManager.h"
#include "Exception.h"
#include "Scripting.h"

#include "OgreOctreePlugin.h"
#include "OgreD3D9Plugin.h"
#include "OgreD3D10Plugin.h"
#include "OgreD3D11Plugin.h"
#include "OgreCgPlugin.h"
#include "OgreGLPlugin.h"
#include "OgreParticleFXPlugin.h"
#include "OgreOggSoundPlugin.h"

#ifdef USE_PARTICLE_UNIVERSE
#include "ParticleUniversePlugin.h"
#endif

namespace SLK
{

class SerilynExport ResourceInitializer
{
public:
	ResourceInitializer(std::string location, std::string type, std::string group)
	{
		this->location = location;
		this->type = type;
		this->group = group;
	}

	std::string location;
	std::string type;
	std::string group;
};

namespace LibFlags
{
	enum Flags
	{
		None = 0x0,
		OgreOctreeSceneManager = 0x001, //One scene manager must be included which means this is required
		OgreCGShaders = 0x002,
		OgreParticleFX = 0x004,
		ParticleUniverse = 0x008, //If ParticleUnverise and ParticleFX are both active, Universe takes precidence
		AngelScript = 0x010,
		Bullet = 0x020,
		OgreOggSound = 0x040,
		SQLite = 0x080,
		OgreRendererOpenGL = 0x100, //At least one of these renderers must be included
		OgreRendererDX9 = 0x200,
		OgreRendererDX10 = 0x400, //Note: This is experimental in Ogre currently
		OgreRendererDX11 = 0x800, //Note: This is experimental in Ogre currently
		Everything = -1
	};
	const int FlagsDefaultRenderers = OgreRendererOpenGL | OgreRendererDX9;
	const int FlagsCompo = OgreOctreeSceneManager | FlagsDefaultRenderers;
	const int FlagsDefault = FlagsCompo;
}

class SerilynExport Application:public Singleton<Application>
{
public:
	Application();
	virtual ~Application(void);

	virtual bool init(unsigned int flags, State *initialState, std::string title, std::vector<ResourceInitializer> &resources, bool quitOnESC);
	virtual bool run();
	virtual void initResources(){};//Initialize any resources here

	UIManager *getInput(){return m_input;};
	Ogre::SceneManager *getSceneManager(){return m_sceneManager;};
	Ogre::Camera *getCamera(){return m_camera;};
	Ogre::Viewport *getViewport(){return m_viewport;};

	SoundManager *getSoundManager(){if(!m_soundManager)throw SLK::Exception("Sound flag not turned on.");return m_soundManager;};

	static Application* getSingletonPtr();
	static Application& getSingleton();
private:
	bool KeyboardDown(SLK::EventArgs &args);
	void DestroyAllChildrenOfNode(Ogre::SceneNode *node);

	Ogre::Root *m_ogreRoot;
	Ogre::RenderWindow *m_window;
	Ogre::SceneManager *m_sceneManager;
	Ogre::Viewport *m_viewport;
	Ogre::Camera *m_camera;

	UIManager *m_input;

	SoundManager *m_soundManager;
	Scripting *m_scripting;

	Timer m_timer;

	bool m_running;

	State *m_state;
	unsigned int m_libFlags;
	//Ogre Plugins
#ifdef OGRE_STATIC_LIB
	Ogre::GLPlugin *m_glPlugin;
	Ogre::D3D9Plugin *m_d3d9Plugin;
	Ogre::D3D10Plugin *m_d3d10Plugin;
	Ogre::D3D11Plugin *m_d3d11Plugin;
	Ogre::OctreePlugin *m_octreePlugin;
	Ogre::CgPlugin *m_cgPlugin;
	Ogre::ParticleFXPlugin *m_particleFXPlugin;
#endif
#ifdef USE_PARTICLE_UNIVERSE
	ParticleUniverse::ParticleUniversePlugin *m_particleUniversePlugin;
#endif

};

}
