/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#ifndef HOTKEYMANAGER_H
#define HOTKEYMANAGER_H
#include <OIS\OIS.h>
#include "Event.h"

namespace SLK
{

namespace KeyModifier
{
	enum Modifier {None=0, Shift=1, Ctrl=2, Alt=3};
	enum Bitmask {BitsNone=0, BitsShift=0x1, BitsCtrl=0x2, BitsAlt=0x4};
}

class SerilynExport HotkeyEventArgs:public EventArgs
{
public:
	enum State{Up, Down};
	HotkeyEventArgs(OIS::KeyCode keycode, State state, KeyModifier::Modifier modifier)
	{
		this->keycode = keycode;
		this->state = state;
		this->modifier = modifier;
	}

	OIS::KeyCode keycode;
	State state;
	KeyModifier::Modifier modifier;
};

class SerilynExport MouseEventArgs:public EventArgs
{
public:
	enum State{None, Up, Down};//If state is none then no buttons are down
	MouseEventArgs(OIS::MouseButtonID button, State state, KeyModifier::Modifier, OIS::MouseState mouseState)
	{
		this->button = button;
		this->state = state;
		this->modifier = modifier;
		this->mouseState = mouseState;
	}
	MouseEventArgs(OIS::MouseState mouseState)
	{
		this->button = OIS::MB_Button7;
		this->state = None;
		this->modifier = KeyModifier::None;
		this->mouseState = mouseState;
	}

	OIS::MouseButtonID button;
	State state;
	KeyModifier::Modifier modifier;

	OIS::MouseState mouseState;
};



class SerilynExport HotkeyManager
{
public:
	HotkeyManager(OIS::Keyboard *keyboard);
	virtual ~HotkeyManager(void);

	//modifier must be KC_UNASSIGNED, KC_SHIFT, KC_ALT or KC_CTRL.
	//key can be any key but modifier keys can't have modifiers themselves.
	template<typename T>
	void assignHotkey(bool (T::*function)(EventArgs &eventArgs), T* object, OIS::KeyCode key, KeyModifier::Modifier modifier=KeyModifier::None);
	bool checkHotkey(OIS::KeyCode key, KeyModifier::Modifier modifier=KeyModifier::None);
	void unassignHotkey(OIS::KeyCode key, KeyModifier::Modifier modifier=KeyModifier::None);
	
	template<typename T>
	void assignMouseButton(bool (T::*function)(EventArgs &eventArgs), T* object, OIS::MouseButtonID button, KeyModifier::Modifier modifier=KeyModifier::None);
	bool checkMouseButton(OIS::MouseButtonID button, KeyModifier::Modifier modifier=KeyModifier::None);
	void unassignMouseButton(OIS::MouseButtonID button, KeyModifier::Modifier modifier=KeyModifier::None);

	template<typename T>
	void assignMouseMove(bool (T::*function)(EventArgs &eventArgs), T* object);
	bool checkMouseMove();
	void unassignMouseMove();

	void pressKey(OIS::KeyCode keycode);
	void releaseKey(OIS::KeyCode keycode);
	void moveMouse(OIS::MouseState mouseState);
	void pressMouse(OIS::MouseState mouseState, OIS::MouseButtonID mouseButton);
	void releaseMouse(OIS::MouseState mouseState, OIS::MouseButtonID mouseButton);

private:
	static const int NUM_KEY_ASSIGNMENTS=255;
	static const int NUM_MODIFIERS=4;//add the NONE modifier too
	int m_modifiers;
	Event m_hotkeys[NUM_MODIFIERS][NUM_KEY_ASSIGNMENTS];
	Event m_mouseButtons[NUM_MODIFIERS][OIS::MB_Button7+1];
	Event m_mouseEvent;

	OIS::Keyboard *m_keyboard;
};

template<typename T>
void HotkeyManager::assignHotkey(bool (T::*function)(EventArgs &eventArgs), T* object, OIS::KeyCode key, KeyModifier::Modifier modifier)
{
	assert(key<NUM_KEY_ASSIGNMENTS);
	m_hotkeys[modifier][key].connect(function,object);

	_CrtCheckMemory();
}

template<typename T>
void HotkeyManager::assignMouseButton(bool (T::*function)(EventArgs &eventArgs), T* object, OIS::MouseButtonID button, KeyModifier::Modifier modifier)
{
	m_mouseButtons[modifier][button].connect(function,object);

	_CrtCheckMemory();
}

template<typename T>
void HotkeyManager::assignMouseMove(bool (T::*function)(EventArgs &eventArgs), T* object)
{
	m_mouseEvent.connect(function,object);
}




}


#endif //HOTKEYMANAGER_H