/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#pragma once
#include "MersenneTwister.h"

namespace SLK
{

static const float invIntMax = 1.0f/(float)RAND_MAX;
static const float PI = 3.14159f;
static const float PI2 = 2* PI;
static const float SQRT_2PI = sqrt(PI2);

inline float getRandBetween(float min, float max)
{
	return ((float)rand()/ ((float)RAND_MAX+1) )*(max-min)+min;
}

inline int rollDice(int dice, int sides)
{
	int num = 0;
	for(int i=0; i<dice; i++)
	{
		num += rand()%sides+1;
	}

	return num;
}

inline void swap(float &x, float &y)
{
    float tmp = x;
    x = y;
    y = tmp;
}

inline int round(double val)
{
	return int(val>0?val+0.5:val-0.5);
}

inline int round(float val)
{
	return int(val>0?val+0.5:val-0.5);
}

//Class for generating bell curves
class SerilynExport NormalGen
{
public:

	//uses stdDev1 on the left or the mean and stdDev2 on the right
	NormalGen(float stdDev,bool useMeanStdDev,int maxY, MTRand *mersenne):
		m_mersenne(mersenne)
	{
		calcSamples(stdDev,useMeanStdDev,maxY);
	}

	int getRandom(int mean, int cur)
	{
		if(cur<0 || cur>MAX_SAMPLES)
			throw std::exception("Error computing normal value.");
		return m_samples[mean][cur];
	}

	int getRandom(int mean)
	{
		int x = m_mersenne->getBetween(0,MAX_SAMPLES);
		return m_samples[mean][x];
	}
private:
	inline float getNormalDistribution(float mean, float stdDev,float x)
	{
		return (1/(stdDev*SQRT_2PI))*exp(-((x-mean)*(x-mean))/(2*stdDev*stdDev));
	}

	//samples are used to bring the max result up to max.
	void calcSamples(float stdDev,bool useMeanStdDev,int maxY)
	{
		float maxSampleL;
		float maxSampleR;

		for(int mean=0; mean<MAX_SAMPLES; mean++)
		{
			float fMean = (float)mean;
			maxSampleL = maxY/getNormalDistribution(fMean,stdDev,fMean);
			maxSampleR = maxY/getNormalDistribution(fMean,fMean,fMean);

			for(int x=0; x<MAX_SAMPLES; x++)
			{
				if(useMeanStdDev && x<mean)
				{
					m_samples[mean][x] = (int)(maxSampleR*getNormalDistribution(fMean,fMean,(float)x));
					if(m_samples[mean][x]<1)
						m_samples[mean][x]=1;
				}
				else
				{
					m_samples[mean][x] =(int)(maxSampleL*getNormalDistribution(fMean,stdDev,(float)x));
					if(m_samples[mean][x]<1)
						m_samples[mean][x]=1;
				}
			}
		}
	}

	static const int MAX_SAMPLES=100;
	int m_samples[MAX_SAMPLES][MAX_SAMPLES];//stored as mean, x
	MTRand *m_mersenne;
};

}