/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#pragma once

#include <Angelscript.h>
#include "Singleton.h"
#include <string>
#include "Script.h"
#include "Exception.h"

namespace SLK
{

	class DebugConsole;

class Scripting:public Singleton<Scripting>
{
public:
	Scripting();
	~Scripting(void);
	
	Script *GetScript(std::string id);
	Script *GetScript(std::string id, std::string scriptString);

	void executeString(std::string &code);
	void executeFile(std::string &fileName);

	static void staticExecuteFile(std::string &fileName)
	{
		Scripting::getSingleton().executeFile(fileName);
	}


	//pass-throughs for registration
	virtual int SetMessageCallback(const asSFuncPtr &callback, void *obj, asDWORD callConv)
	{
		int r = m_engine->SetMessageCallback(callback,obj,callConv);
		if(r < 0) throw Exception("Error registering scripting message callback (%s)",errorCodeToString(r).c_str());
		return r;
	}

	virtual int ClearMessageCallback()
	{
		int r = m_engine->ClearMessageCallback();
		if(r < 0) throw Exception("Error unregistering scripting message callback (%s)",errorCodeToString(r).c_str());
		return r;
	}

	inline int RegisterObjectType(const char *name, int byteSize, asDWORD flags)
	{
		int r = m_engine->RegisterObjectType(name,byteSize,flags);
		if(r < 0)throw Exception("Error registering class (%s) for scripting (%s)",name,errorCodeToString(r).c_str());
		return r;
	}

	inline int RegisterObjectProperty(const char *obj, const char *declaration, int byteOffset)
	{
		int r = m_engine->RegisterObjectProperty(obj,declaration,byteOffset);
		if(r < 0)throw Exception("Error registering class property (%s %s) for scripting (%s)",obj,declaration,errorCodeToString(r).c_str());
		return r;
	}

	inline int RegisterObjectMethod(const char *obj, const char *declaration, const asSFuncPtr &funcPointer, asDWORD callConv)
	{
		int r = m_engine->RegisterObjectMethod(obj,declaration,funcPointer,callConv);
		if(r < 0)throw Exception("Error registering class method (%s %s) for scripting (%s)",obj,declaration,errorCodeToString(r).c_str());
		return r;	
	}

	inline int RegisterObjectBehaviour(const char *obj, asEBehaviours behaviour, const char *declaration, const asSFuncPtr &funcPointer, asDWORD callConv)
	{
		int r = m_engine->RegisterObjectBehaviour(obj,behaviour,declaration,funcPointer,callConv);
		if(r < 0)throw Exception("Error registering class behaviour (%s %s) for scripting (%s)",obj,declaration,errorCodeToString(r).c_str());
		return r;	
	}

	inline int RegisterGlobalProperty(const char *declaration, void *pointer)
	{
		int r = m_engine->RegisterGlobalProperty(declaration,pointer);
		if(r < 0)throw Exception("Error registering property (%s) for scripting (%s)",declaration,errorCodeToString(r).c_str());
		return r;	
	}

	inline int RegisterGlobalFunction(const char *declaration, const asSFuncPtr &funcPointer, asDWORD callConv)
	{
		int r = m_engine->RegisterGlobalFunction(declaration,funcPointer,callConv);
		if(r < 0)throw Exception("Error registering function (%s) for scripting (%s)",declaration,errorCodeToString(r).c_str());
		return r;	
	}

	inline int RegisterInterface(const char *name)
	{
		int r = m_engine->RegisterInterface(name);
		if(r < 0)throw Exception("Error registering interface (%s) for scripting (%s)",name,errorCodeToString(r).c_str());
		return r;	
	}

	inline int RegisterInterfaceMethod(const char *intf, const char *declaration)
	{
		int r = m_engine->RegisterInterfaceMethod(intf,declaration);
		if(r < 0)throw Exception("Error registering interface method (%s %s) for scripting (%s)",intf,declaration,errorCodeToString(r).c_str());
		return r;	
	}

	inline int RegisterEnum(const char *type)
	{
		int r = m_engine->RegisterEnum(type);
		if(r < 0)throw Exception("Error registering enum (%s) for scripting (%s)",type,errorCodeToString(r).c_str());
		return r;	
	}

	inline int RegisterEnumValue(const char *type, const char *name, int value)
	{
		int r = m_engine->RegisterEnumValue(type,name,value);
		if(r < 0)throw Exception("Error registering enum  value (%s %s) for scripting (%s)",type,name,errorCodeToString(r).c_str());
		return r;	
	}

	inline int RegisterTypedef(const char *type, const char *decl)
	{
		int r = m_engine->RegisterTypedef(type,decl);
		if(r < 0)throw Exception("Error registering typedef (%s %s) for scripting (%s)",type,decl,errorCodeToString(r).c_str());
		return r;	
	}

	inline int RegisterStringFactory(const char *datatype, const asSFuncPtr &factoryFunc, asDWORD callConv)
	{
		int r = m_engine->RegisterStringFactory(datatype,factoryFunc,callConv);
		if(r < 0)throw Exception("Error registering string factory (%s) for scripting (%s)",datatype,errorCodeToString(r).c_str());
		return r;	
	}

	//Class casting
	template<class A, class B>
	static inline B* refCast(A* a)
	{
		// If the handle already is a null handle, then just return the null handle
		if( !a ) return 0;

		// Now try to dynamically cast the pointer to the wanted type
		B* b = dynamic_cast<B*>(a);
		if( b != 0 )
		{
			// Since the cast was made, we need to increase the ref counter for the returned handle
			//b->addref();//Using automatic ref counting
		}
		return b;

	}

	asIScriptEngine *getEngine(){return m_engine;};

private:
	static std::string errorCodeToString(int errorCode);

	asIScriptEngine *m_engine;

	std::map<std::string, Script*> m_scripts;

};

}
