/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/
#include "StdAfx.h"

#include "DBRow.h"
#include "Exception.h"
#include "Serializer.h"

namespace SLK
{

DBRow::DBRow(sqlite3_stmt *statement)
{
	int count = sqlite3_column_count(statement);
	for(int i=0; i<count; i++)
	{
		std::string name = sqlite3_column_name(statement,i);
		int type = sqlite3_column_type(statement,i);

		DBValue *dbVal = 0;
		switch(type)
		{
		case SQLITE_INTEGER:
			{
			int val = sqlite3_column_int(statement,i);
			dbVal = new DBValueType<int>(val,type);
			}
			break;
		case SQLITE_FLOAT:
			{
			float val = sqlite3_column_double(statement,i);
			dbVal = new DBValueType<double>(val,type);
			}
			break;
		case SQLITE_TEXT:
			{
			const unsigned char *val = sqlite3_column_text(statement,i);
			std::stringstream stream;
			stream << val;
			dbVal = new DBValueType<std::string>(stream.str(),type);
			}
			break;
		case SQLITE_BLOB:
			{
			int size = sqlite3_column_bytes(statement,i);
			const void *val = sqlite3_column_blob(statement,i);
			Serializer *serializer = new Serializer((char*)val, size);
			dbVal = new DBValueType<Serializer*>(serializer,type);
			}
		}

		m_values.insert(std::make_pair<std::string,DBValue*>(name,dbVal));

	}
}

DBRow::~DBRow(void)
{
	for(ValueMap::iterator iter = m_values.begin(); iter!=m_values.end(); iter++)
	{
		SAFEDELETE((*iter).second);
	}	
}

void DBRow::Get(std::string name, std::string &value)
{
	ValueMap::iterator iter = m_values.find(name);
	if(iter == m_values.end())
		throw SLK::Exception("Column %s not in database",name.c_str());
	if(iter->second->getDataType() == SQLITE3_TEXT)
	{
		DBValueType<std::string> *val = (DBValueType<std::string>*)(iter->second);
		value = val->getValue();
	}
	else
	{
		value = iter->second->getAsString();
	}
}

std::ostream& operator<< (std::ostream& o, const Serializer& blob)
{
	return o << blob.getDataSize() << " byte blob";
}


}
