/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/
#include "StdAfx.h"


#include "DBStatement.h"
#include "exception.h"
#include "Database.h"


namespace SLK
{

DBStatement::DBStatement(sqlite3 *database):
	m_statement(0),
	m_database(database)
{

}

DBStatement::DBStatement(Database *database):
	m_statement(0),
	m_database(database->getSQLite())
{

}

DBStatement::~DBStatement(void)
{
	clear();
}

void DBStatement::prepare(std::string sqlStatement)
{
	clear();

	const char *statementSQL=sqlStatement.c_str();
	int size = sqlStatement.length() * sizeof(std::string::value_type)+1;
	int returnVal = sqlite3_prepare_v2(m_database,statementSQL,size,&m_statement,&statementSQL);
	if(returnVal != SQLITE_OK)
	{
		std::string error = sqlite3_errmsg(m_database);
		m_statement = 0;
		throw SLK::Exception("%s String: %s", error.c_str(),sqlStatement.c_str());
	}
	assert(*statementSQL==0 && "WARNING. Multiple statements not run");
}

void DBStatement::prepareAndRun(std::string sqlStatement)
{
	clear();

	const char *statementSQL=sqlStatement.c_str();
	do
	{
		int size = (sqlStatement.length()+1) * sizeof(std::string::value_type);
		int returnVal = sqlite3_prepare_v2(m_database,statementSQL,size,&m_statement,&statementSQL);
		if(returnVal != SQLITE_OK)
		{
			std::string error = sqlite3_errmsg(m_database);
			m_statement = 0;
			throw SLK::Exception("%s String: %s", error.c_str(),sqlStatement.c_str());
			//throw exception("Error preparing statement. ErrorCode: %d",returnVal);
		}
		run();
		clear();
	}while(*statementSQL!=0);
}

void DBStatement::bind(std::string name, float val)
{
	int returnVal = sqlite3_bind_double(m_statement,getParamIndex(name),(double)val);
	if(returnVal != SQLITE_OK)
	{
		std::string error = sqlite3_errmsg(m_database);
		m_statement = 0;
		throw SLK::Exception(error.c_str());
	}
}

void DBStatement::bind(std::string name, double val)
{
	int returnVal = sqlite3_bind_double(m_statement,getParamIndex(name),val);
	if(returnVal != SQLITE_OK)
	{
		std::string error = sqlite3_errmsg(m_database);
		m_statement = 0;
		throw SLK::Exception(error.c_str());
	}
}

void DBStatement::bind(std::string name, __int64 val)
{
	int returnVal = sqlite3_bind_int64(m_statement,getParamIndex(name),val);
	if(returnVal != SQLITE_OK)
	{
		std::string error = sqlite3_errmsg(m_database);
		m_statement = 0;
		throw SLK::Exception(error.c_str());
	}
}

void DBStatement::bind(std::string name, int val)
{
	int returnVal = sqlite3_bind_int(m_statement,getParamIndex(name),val);
	if(returnVal != SQLITE_OK)
	{
		std::string error = sqlite3_errmsg(m_database);
		m_statement = 0;
		throw SLK::Exception(error.c_str());
	}
}

void DBStatement::bind(std::string name, std::string val)
{
	int size = val.length() * sizeof(std::string::value_type);
	int returnVal = sqlite3_bind_text(m_statement,getParamIndex(name),val.c_str(),size,SQLITE_TRANSIENT);
	if(returnVal != SQLITE_OK)
	{
		std::string error = sqlite3_errmsg(m_database);
		m_statement = 0;
		throw SLK::Exception(error.c_str());
	}
}

void DBStatement::bind(std::string name, Serializer &serializer)
{
	bind(name, serializer.getData(), serializer.getDataSize());
}

void DBStatement::bind(std::string name, void* blob, int size)
{
	int returnVal = sqlite3_bind_blob(m_statement,getParamIndex(name),blob,size,SQLITE_STATIC);
	if(returnVal != SQLITE_OK)
	{
		std::string error = sqlite3_errmsg(m_database);
		m_statement = 0;
		throw SLK::Exception(error.c_str());
	}
}

void DBStatement::run(DBResultSet* results)
{
	int result;
	do
	{
		result = sqlite3_step(m_statement);
		switch(result)
		{
		case SQLITE_BUSY:
			//do something?
			assert(false && "Database busy");
			break;
		case SQLITE_ERROR:
			{
			std::string error = sqlite3_errmsg(m_database);
			m_statement = 0;
			throw SLK::Exception(error.c_str());
			}
			break;
		case SQLITE_MISUSE:
			m_statement = 0;
			throw SLK::Exception("Database statement can not be run at this time.");
			break;
		case SQLITE_ROW:
			if(results)
				results->AddRow(m_statement);
			break;
		}
	}while(result==SQLITE_ROW);
}

void DBStatement::reset()
{
	int returnVal = sqlite3_reset(m_statement);
	if(returnVal != SQLITE_OK)
	{
		std::string error = sqlite3_errmsg(m_database);
		m_statement = 0;
		throw SLK::Exception(error.c_str());
	}
}

void DBStatement::clear()
{
	if(m_statement)
	{
		int returnVal = sqlite3_finalize(m_statement);
		if(returnVal != SQLITE_OK)
		{
			std::string error = sqlite3_errmsg(m_database);
			m_statement = 0;
			throw SLK::Exception(error.c_str());
		}
		m_statement = 0;
	}
}

int DBStatement::getParamIndex(std::string paramName)
{
	int index = sqlite3_bind_parameter_index(m_statement,paramName.c_str());
	if(index==0)
	{
		m_statement = 0;
		throw SLK::Exception("No param found with name %s",paramName.c_str());
	}
	return index;
}


}
