/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2010 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

#include "StdAfx.h"

#include "Scripting.h"

#include "DebugConsole.h"

#include "OgreScriptSupport.h"
#include "Handle.h"
#include "MersenneTwister.h"
#include "random.h"
#include "../Libs/Angelscript/sdk/add_on/scripthelper/scripthelper.h"
#include "../Libs/Angelscript/sdk/add_on/scriptstring/scriptstring.h"
#include "../Libs/Angelscript/sdk/add_on/scriptstdstring/scriptstdstring.h"

namespace SLK
{

// Implement a simple message callback function
void MessageCallback(const asSMessageInfo *msg, void *param)
{
	const char *type = 0;
	Ogre::LogMessageLevel level;
	switch(msg->type)
	{
	case asMSGTYPE_ERROR:
		type = "ERROR";
		level = Ogre::LML_CRITICAL;
		break;
	case asMSGTYPE_WARNING:
		type = "WARNING";
		level = Ogre::LML_NORMAL;
		break;

	case asMSGTYPE_INFORMATION:
		type = "INFORMATION";
		level = Ogre::LML_TRIVIAL;
		break;
	}
	std::stringstream str;
	str << msg->section << " (" << msg->row << ", " << msg->col << ") : " << type << " : " << msg->message;
	Ogre::LogManager::getSingletonPtr()->logMessage(str.str(), level);

};

static void HandleDefaultConstructor(Handle *self)
{
	new(self) Handle();
}

Scripting::Scripting():
	m_engine(0)
{
	m_engine = asCreateScriptEngine(ANGELSCRIPT_VERSION);
	m_engine->SetMessageCallback(asFUNCTION(MessageCallback), 0, asCALL_CDECL);
	if(!m_engine)
	{
		throw std::exception("Could not initialize Angelscript");
	}

	RegisterStdString(m_engine);
	OgreScriptSupport::RegisterVector2();
	OgreScriptSupport::RegisterVector3();

	//Register mersenne twister
	Scripting::getSingleton().RegisterObjectType("Random", 0, asOBJ_REF | asOBJ_NOHANDLE);

	Scripting::getSingleton().RegisterObjectMethod("Random","double rand()", asMETHODPR(MTRand,rand,(),double), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("Random","double rand(const double)", asMETHODPR(MTRand,rand,(const double),double), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("Random","uint randInt()", asMETHODPR(MTRand,randInt,(),MTRand::uint32), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("Random","uint randInt(const uint)", asMETHODPR(MTRand,randInt,(const MTRand::uint32),MTRand::uint32), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("Random","double getBetween(double, double)", asMETHODPR(MTRand,getBetween,(double, double),double), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("Random","int getBetween(int, int)", asMETHODPR(MTRand,getBetween,(int,int),int), asCALL_THISCALL);
	Scripting::getSingleton().RegisterObjectMethod("Random","uint getBetween(uint, uint)", asMETHODPR(MTRand,getBetween,(unsigned int, unsigned int),unsigned int), asCALL_THISCALL);

	// Regisister Handle
	Scripting::getSingleton().RegisterObjectType("Handle", sizeof(Handle), asOBJ_VALUE | asOBJ_POD | asOBJ_APP_CLASS_CA);
	SLK::Scripting::getSingleton().RegisterObjectMethod("Handle","bool isValid()", asMETHOD(Handle,isValid), asCALL_THISCALL);
	// Register Handle operator overloads
	Scripting::getSingleton().RegisterObjectBehaviour("Handle", asBEHAVE_CONSTRUCT,  "void Handle()",  asFUNCTION(HandleDefaultConstructor), asCALL_CDECL_OBJLAST);

	RegisterGlobalFunction("void RunScript(const string &in)",asFUNCTION(Scripting::staticExecuteFile),asCALL_CDECL);

}

Scripting::~Scripting(void)
{
	for(std::map<std::string, Script*>::iterator iter = m_scripts.begin(); iter!= m_scripts.end(); iter++)
	{
		SAFEDELETE(iter->second);
	}

	if (m_engine) 
	{ 
		m_engine->Release(); 
		m_engine = 0; 
	}
}

/*
 id = filename or id. If script doesn't exist, will try to load id as filename
*/
Script *Scripting::GetScript(std::string id)
{
	std::map<std::string, Script*>::iterator iter = m_scripts.find(id);
	if(iter !=m_scripts.end())
		return iter->second;
	Script *script = new Script(id);
	if(!script->isOK())
	{
		assert(false && "Script did not load");
		SAFEDELETE(script);
	}
	else
	{
		m_scripts.insert(std::make_pair(id, script));
	}

	return script;
}

Script *Scripting::GetScript(std::string id, std::string scriptString)
{
	std::map<std::string, Script*>::iterator iter = m_scripts.find(id);
	if(iter !=m_scripts.end())
		return iter->second;
	Script *script = new Script(id, scriptString);
	if(!script->isOK())
	{
		assert(false && "Script did not load");
		SAFEDELETE(script);
	}
	else
	{
		m_scripts.insert(std::make_pair(id, script));
	}

	return script;
}

void Scripting::executeString(std::string &code)
{
	int r = -1;
	
	if(code.size()>0 && code[code.size()-1] != '\'')
		r = ExecuteString(m_engine, code.c_str());

	if(r < 0)
		DebugConsole::getSingleton().addText("Invalid command.");
	else if(r == asEXECUTION_EXCEPTION)
		DebugConsole::getSingleton().addText("A script exception was raised.");
}

void Scripting::executeFile(std::string &fileName)
{
	Script *script = new Script(fileName);
	if(script->isOK())
	{
		script->prepareAndRun();
	}
	SAFEDELETE(script);

}

std::string Scripting::errorCodeToString(int errorCode)
{
	switch(errorCode)
	{
	case asSUCCESS:
		return "Success";
	case asERROR:
		return "Failure";
	case asCONTEXT_ACTIVE:
		return "The context is active";
	case asCONTEXT_NOT_FINISHED:
		return "The context is not finished";
	case asCONTEXT_NOT_PREPARED:
		return "The context is not prepared";
	case asINVALID_ARG:
		return "Invalid argument";
	case asNO_FUNCTION:
		return "The function was not found";
	case asNOT_SUPPORTED:
		return "Not supported";
	case asINVALID_NAME:
		return "Invalid name";
	case asNAME_TAKEN:
		return "The name is already taken";
	case asINVALID_DECLARATION:
		return "Invalid declaration";
	case asINVALID_OBJECT:
		return "Invalid object";
	case asINVALID_TYPE:
		return "Invalid type";
	case asALREADY_REGISTERED:
		return "Already registered";
	case asMULTIPLE_FUNCTIONS:
		return "Multiple matching functions";
	case asNO_MODULE:
		return "The module was not found";
	case asNO_GLOBAL_VAR:
		return "The global variable was not found";
	case asINVALID_CONFIGURATION:
		return "Invalid configuration";
	case asINVALID_INTERFACE:
		return "Invalid interface";
	case asCANT_BIND_ALL_FUNCTIONS:
		return "All imported functions couldn't be bound";
	case asLOWER_ARRAY_DIMENSION_NOT_REGISTERED:
		return "The array sub type has not been registered yet";
	case asWRONG_CONFIG_GROUP:
		return "Wrong configuration group";
	case asCONFIG_GROUP_IS_IN_USE:
		return "The configuration group is in use";
	case asILLEGAL_BEHAVIOUR_FOR_TYPE:
		return "Illegal behaviour for the type";
	case asWRONG_CALLING_CONV:
		return "The specified calling convention doesn't match the function/method pointer";
	case asBUILD_IN_PROGRESS:
		return "A build is currently in progress";
	case asINIT_GLOBAL_VARS_FAILED:
		return "Initializing global variables failed";
	}

	std::stringstream str;
	str << "Unknown (" << errorCode <<")";

	return str.str();
}

}
